<?php

class Payzoff_Mygateway_Model_Standard extends Mage_Payment_Model_Method_Abstract
{
    protected $_code = 'mygateway';
    protected $_isInitializeNeeded = true;
    protected $_canUseInternal = false;
    protected $_canUseForMultishipping = false;

    /**
     * Redirect to payzoff after receiving information from payzoff
     *
     * @return array
     */
    public function getOrderPlaceRedirectUrl()
    {
        $return_url = $this->processPayzoff();

        return $return_url;
    }

    /**
     * Process payzoff and preparing data.
     *
     * @return array
     */
    public function processPayzoff()
    {
        // Getting the total amount of the cart after all discounts and taxes
        $quote     = Mage::getModel( 'checkout/session' )->getQuote();
        $quoteData = $quote->getData();
        $amount    = $quoteData['grand_total'];
        $quoteId   = Mage::getSingleton( 'checkout/session' )->getQuoteId();

        $return_url    = Mage::getBaseUrl() . 'mygateway/payment/return';
        $result_url    = Mage::getBaseUrl() . 'mygateway/payment/result';
        $fail_url      = Mage::getBaseUrl() . 'mygateway/payment/fail';
        $currency_code = Mage::app()->getStore()->getCurrentCurrencyCode();

        $name = Mage::getSingleton( 'customer/session' )->getCustomer()->getName();
        if ( empty($name) )
        {
            $name = 'Guest User';
        }

        $cartItems = Mage::getSingleton('checkout/session')->getQuote()->getAllItems();
        $description  = Array();
        foreach($cartItems as $item)
        {
            $custom = Mage::getModel('catalog/product')->load($item->getProduct()->getId());
            $description[] = $custom->getShortDescription();
        }

        $args = Array(
            'action'                => 'payment',
            'return_url'            => $return_url,
            'result_url'            => $result_url,
            'fail_url'              => $fail_url,
            'amount'                => $amount,
            'currency'              => !empty($currency_code) ? $currency_code : Mage::app()->getStore()->getDefaultCurrencyCode(),
            'seller_transaction_id' => $quoteId,
            'description'           =>  implode(',', $description),
            'name'                  => $name,
        );

        $api_key    = Mage::getStoreConfig( 'payment/mygateway/payzoff_api' );
        $secret_key = Mage::getStoreConfig( 'payment/mygateway/payzoff_secret' );
        $test_mode  = Mage::getStoreConfig( 'payment.mygateway/payzoff_mode' );

        $signature = md5( $api_key . $args['amount'] . $args['seller_transaction_id'] . $secret_key );

        $args += Array(
            'api_key'          => $api_key,
            'signature'        => $signature,
            'transaction_mode' => $test_mode
        );


        $tfile = fopen( 'payzoff-standard.txt', 'w' );
        fwrite( $tfile, print_r( $args, true ) );
        fclose($tfile);

        $data = (array)$this->callPayzoff( $args );


        $tfile = fopen( 'payzoff-data.txt', 'w' );
        fwrite( $tfile, print_r( $data, true ) );
        fclose($tfile);
        

        if ( $data['status_code'] == 0 && $data['status'] == 'ok' )
        {
            $return_url     = $data['result']['redirect_url'];
            $transaction_id = $data['result']['transaction_id'];
            $secret_code    = $data['result']['secret_code'];

            // Saving infomation in database for reference
            $db = Mage::getModel( 'mygateway/transaction' );
            $db->setQuoteId( $quoteId );
            $db->setPayoffTransactionId( $transaction_id );
            $db->setSecretCode( $secret_code );
            $db->save();

            return $return_url;
        }
        else
        {
            $helper    = Mage::helper( 'mygateway/data' );
            $error_msg = $this->getErrorMessage( $data['status_code'] );

            Mage::throwException( $error_msg );
        }
    }

    /**
     * Making request to payzoff API and processing the information
     *
     * @param $args
     *
     * @return mixed
     */
    private function callPayzoff( $args )
    {
        $api_url = Mage::getStoreConfig( 'payment/mygateway/submit_url' );

        $ch = curl_init( $api_url );

        curl_setopt( $ch, CURLOPT_SSL_VERIFYPEER, FALSE );
        curl_setopt( $ch, CURLOPT_SSL_VERIFYHOST, FALSE );
        curl_setopt( $ch, CURLOPT_POST, true );
        curl_setopt( $ch, CURLOPT_POSTFIELDS, $args );
        curl_setopt( $ch, CURLOPT_RETURNTRANSFER, true );
        curl_setopt( $ch, CURLOPT_CONNECTTIMEOUT, 3 );
        curl_setopt( $ch, CURLOPT_TIMEOUT, 20 );

        $response = curl_exec( $ch );

        curl_close( $ch );

        return json_decode( $response, true );
    }

}